#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.listeners.ilistener import IListener
from abc import ABCMeta

class AbstractListener(IListener):
    '''
    Abstract listener, listens for the events
    '''

    __metaclass__ = ABCMeta

    def __init__(self):
        self.__id = id(self)



#
# '''
# Created on Oct 22, 2013
#
# @author: mpastern
# '''
#
import curses


# from ovirtcli.state.finitestatemachine import FiniteStateMachine
# from ovirtcli.state.dfsaevent import DFSAEvent
# from ovirtcli.state.dfsastate import DFSAState
#
# import curses, traceback
#
# class Test(object):
#     '''
#     the test class
#     '''
#     def run(self):
# PColLinux =  coloransi.ColorScheme(
#     'Linux',
#     in_prompt  = InputColors.Green,
#     in_number  = InputColors.LightGreen,
#     in_prompt2 = InputColors.Green,
#     in_normal  = InputColors.Normal,  # color off (usu. Colors.Normal)
#
#     out_prompt = Colors.Red,
#     out_number = Colors.LightRed,
#
#     normal = Colors.Normal
#     )
# #         sm = FiniteStateMachine(
# #             events=[
# #                 DFSAEvent(
# #                   name='disconnect',
# #                   sources=[
# #                        DFSAState.CONNECTED,
# #                        DFSAState.UNAUTHORIZED
# #                   ],
# #                   destination=DFSAState.Disconnected,
# #                   callbacks=[self.onDisconnect]),
# #                 DFSAEvent(
# #                   name='connect',
# #                   sources=[
# #                        DFSAState.Disconnected,
# #                        DFSAState.Unauthorized
# #                   ],
# #                   destination=DFSAState.Connected,
# #                   callbacks=[self.onConnect]),
# #                 DFSAEvent(
# #                   name='unauthorized',
# #                   sources=[
# #                        DFSAState.Connected
# #                   ],
# #                   destination=DFSAState.Unauthorized,
# #                   callbacks=[]),
# #             ]
# #         )
#
# #         sm2 = FiniteStateMachine(
# #             events=[
# #                 DFSAEvent(
# #                   name='disconnect',
# #                   sources=[
# #                        DFSAState.Connected,
# #                        DFSAState.Unauthorized
# #                   ],
# #                   destination=DFSAState.Disconnected,
# #                   callbacks=[self.onDisconnect]),
# #                 DFSAEvent(
# #                   name='connect',
# #                   sources=[
# #                        DFSAState.Disconnected,
# #                        DFSAState.Unauthorized
# #                   ],
# #                   destination=DFSAState.Connected,
# #                   callbacks=[self.onConnect]),
# #                 DFSAEvent(
# #                   name='unauthorized',
# #                   sources=[
# #                        DFSAState.Connected
# #                   ],
# #                   destination=DFSAState.Unauthorized,
# #                   callbacks=[]),
# #             ]
# #         )
# #
# #         print sm is sm2
# #         print id(sm) == id(sm2)
#
# #         sm.connect()
# #         print sm; print "\n"
# #         sm.disconnect()
# #         print sm; print "\n"
# #         sm.unauthorized()
# #         print sm; print "\n"
#
#
# #     def onConnect(self, **kwargs):
# #         print "connect callback with:\n%s\n\n" % kwargs['event']
# #
# #     def onDisconnect(self, **kwargs):
# #         print "disconnect callback:\n%s\n\n" % kwargs['event']
#
# Test().run()
# import curses
# from curses import panel
#
# class Menu(object):
#
#     def __init__(self, items, stdscreen):
#         self.window = stdscreen.subwin(0, 0)
#         self.window.keypad(1)
#         self.panel = panel.new_panel(self.window)
#         self.panel.hide()
#         panel.update_panels()
#
#         self.position = 0
#         self.items = items
#         self.items.append(('exit', 'exit'))
#
#     def navigate(self, n):
#         self.position += n
#         if self.position < 0:
#             self.position = 0
#         elif self.position >= len(self.items):
#             self.position = len(self.items) - 1
#
#     def display(self):
#         self.panel.top()
#         self.panel.show()
#         self.window.clear()
#
#         while True:
#             self.window.refresh()
#             curses.doupdate()
#             for index, item in enumerate(self.items):
#                 if index == self.position:
#                     mode = curses.A_REVERSE
#                 else:
#                     mode = curses.A_NORMAL
#
#                 msg = '%d. %s' % (index, item[0])
#                 self.window.addstr(1 + index, 1, msg, mode)
#
#             key = self.window.getch()
#
#             if key in [curses.KEY_ENTER, ord('\n')]:
#                 if self.position == len(self.items) - 1:
#                     break
#                 else:
#                     self.items[self.position][1]()
#
#             elif key == curses.KEY_UP:
#                 self.navigate(-1)
#
#             elif key == curses.KEY_DOWN:
#                 self.navigate(1)
#
#         self.window.clear()
#         self.panel.hide()
#         panel.update_panels()
#         curses.doupdate()
#
# class MyApp(object):
#
#     def __init__(self, stdscreen):
#         self.screen = stdscreen
#         curses.curs_set(0)
#
#         submenu_items = [
#                 ('beep', curses.beep),
#                 ('flash', curses.flash)
#                 ]
#         submenu = Menu(submenu_items, self.screen)
#
#         main_menu_items = [
#                 ('beep', curses.beep),
#                 ('flash', curses.flash),
#                 ('submenu', submenu.display)
#                 ]c
#         main_menu = Menu(main_menu_items, self.screen)
#         main_menu.display()
#
# if __name__ == '__main__':
#     curses.wrapper(MyApp)

import curses

# initialize curses
stdscr = curses.initscr()
curses.start_color()

# initialize color #1 to Blue with Cyan background
curses.init_pair(1, curses.COLOR_BLUE, curses.COLOR_CYAN)

stdscr.addstr('A sword and a shield.', curses.color_pair(1))
stdscr.refresh()

# finalize curses
curses.endwin()



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import os
import sys

from ovirtcli.options import OvirtCliOptionParser
from ovirtcli.context import OvirtCliExecutionContext
from ovirtcli.object import create

def copy_environment_vars(context):
    """Copy environment variables into configuration variables in the
    execution context."""
    for var in ('url', 'username', 'password'):
        envvar = 'oVirt_%s' % var.upper()
        confvar = 'ovirt-shell:%s' % var
        if envvar in os.environ:
            try:
                context.settings[confvar] = os.environ[envvar]
            except ValueError, e:
                sys.stderr.write('error: %s\n' % str(e))
                return False
    return True


def copy_cmdline_options(options, context, parser):
    """Copy command-line options into configuration variables in the
    execution context."""
    for opt in parser.option_list:
        if not opt.dest:
            continue
        value = getattr(options, opt.dest)
        if value is None:
            continue
        if opt.dest in ('debug', 'verbosity'):
            dest = 'cli:%s' % opt.dest
        else:
            dest = 'ovirt-shell:%s' % opt.dest
        try:
            context.settings[dest] = value
        except KeyError:
            pass
        except ValueError, e:
            sys.stderr.write('error: %s\n' % str(e))
            return False
    return True


def main():
    """Entry point for ovirt-shell."""
    parser = create(OvirtCliOptionParser)
    opts, args = parser.parse_args()

    context = create(OvirtCliExecutionContext)
    if not copy_environment_vars(context):
        sys.exit(1)
    if not copy_cmdline_options(opts, context, parser):
        sys.exit(1)

    if opts.help_commands:
        args = ['help']

    if opts.filter:
        try:
            sys.stdin = file(opts.filter)
        except IOError, e:
            sys.stderr.write('error: %s\n' % e.strerror)
            sys.exit(1)

    if opts.connect or len(args) > 0:
        context.execute_string('connect\n')

    if len(args) == 0:
        context.execute_loop()
    else:
        command = ' '.join(args)
        if opts.read_input:
            buf = sys.stdin.read()
            command += '<<EOM\n%s\nEOM' % buf
        command += '\n'
        context.execute_string(command)
    sys.exit(context.status)

if __name__ == "__main__":
    main()



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.shell.cmdshell import CmdShell
from ovirtcli.utils.typehelper import TypeHelper
from ovirtcli.utils.autocompletionhelper import AutoCompletionHelper
from ovirtsdk.infrastructure import brokers
from ovirtcli.utils.methodhelper import MethodHelper


class ListCmdShell(CmdShell):
    NAME = 'list'
    OPTIONS = ['show-all']

    def __init__(self, context, parser):
        CmdShell.__init__(self, context, parser)

    def do_list(self, args):
        return self.context.execute_string(ListCmdShell.NAME + ' ' + args + '\n')

    def __add_resource_specific_options(self, obj, specific_options, line, key=None):
        typ = TypeHelper.getDecoratorType(obj)
        if typ:
            plur_obj = TypeHelper.to_plural(typ)
            if hasattr(brokers, plur_obj):
                method_args = MethodHelper.getMethodArgs(brokers,
                                                         plur_obj,
                                                         ListCmdShell.NAME,
                                                         True,
                                                         True,
                                                         True)
                if method_args:
                    specific_options[obj if key == None else key] = method_args

    def complete_list(self, text, line, begidx, endidx):
        args = TypeHelper.get_types_by_method(True, ListCmdShell.NAME, expendNestedTypes=True)
        specific_options = self.get_resource_specific_options(args,
                                                              line,
                                                              callback=self.__add_resource_specific_options)
        return AutoCompletionHelper.complete(line,
                                             text,
                                             args=args,
                                             common_options=ListCmdShell.OPTIONS,
                                             specific_options=specific_options)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from ovirtcli.utils.typehelper import TypeHelper

from ovirtsdk.utils.parsehelper import ParseHelper
import types
from cli.messages import Messages

class ActionCommand(OvirtCommand):

    name = 'action'
    description = 'execute an action on an object'
    usage = 'action <type> <id> <action> [parent identifiers] [command options]'
    args_check = 3
    valid_options = [ ('*', str) ]

    helptext0 = """\
        == Usage ==

        action <type> <id> <action> [parent identifiers] [command options]

        == Description ==

        Executes the an action on an object. This command requires the
        following arguments:

          * type        - The type to operate on
          * id          - The name or id identifying the object
          * action      - The action to take

        For more specific help on the available actions and options, use
        'help action <type> <id>'

        == Supported Help formats ==

        - This help will list all available attribute options for given resource
          
          * format      - help action type id
          * example     - help action vm myvm

        - This help will list all available attribute options for specific action on given resource
        
          * format      - help action type id action_name
          * example     - help action vm myvm start
        
        - This help will list all available attribute options for specific subresource 
        
          * format      - help action resource resource_name --subresource-identifier mysubresource
          * example     - help action nic bond0 --host-identifier myhost
        
        - This help will display all available attribute options for specific action on given subresource

          * format      - help action action_name resource resource_name --subresource-identifier mysubresource
          * example     - help action attach nic bond0 --host-identifier myhost

        == Available types ==

        The <type> parameter must be one of the following:

          $types

        == Return values ==

        This command will return one of the following statuses. To see the
        exit status of the last command, type 'status'.

          $statuses
        """

    helptext1 = """\
        == Usage ==

        action <type> <id> <action> [parent identifiers] [command options]

        == Description ==

        Executes the an action on an object. This command requires the
        following arguments:

          * type        - The type to operate on
          * id          - The name or id identifying the object
          * action      - The action to take

        == Available actions ==

        The following actions are available for the specified $type object:

          $actions

        == Collection based option format ==

          * [--x-y: collection]
            {
              [y.a: string]
              [y.b: string]
              [y.c: string]
            }

          e.g:

          --x-y "y.a=a1,y.b=b1,y.c=c1"
          --x-y "y.a=a2,y.b=b2,y.c=c2"
          ...

          where a, b, c are option properties and aN, bN, cN is actual user's data

        == Object Identifiers ==

        Some objects can only exist inside other objects. For example, a disk
        can only exist in the content of a virtual machine. In this case, one
        or more object identifiers needs to be provided to identify the
        containing object.

        An object identifier is an option of the form '--<type>id <id>'. This
        would identify an object with type <type> and id <id>. See the
        examples section below for a few examples.

        == Attribute Options ==

        The following attribute options are understood. Note: this lists all
        available attribute options for action '$action'.

          $options

        == Examples ==

        - This example starts a vm named "myvm" with few optional parameters:

          $ action vm iscsi_desktop start --vm-display-type vnc --vm-cpu-topology-sockets 2 --vm-cpu-topology-cores 2

        - This example detaches a host nic with id 'mynic' from host 'myhost':

          $ action nic mynic detach --host-identifier myhost
    
        == Return values ==

        This command will exit with one of the following statuses. To see the
        exit status of the last command, type 'status'.

          $statuses
        """

    def execute(self):
        """Execute the action command."""
        args = self.arguments
        opts = self.options

        if not (TypeHelper.isKnownType(args[0])) and \
           ParseHelper.getXmlType(args[0]) == None:
            self.error(
                       Messages.Error.NO_SUCH_TYPE % args[0]
            )

        # scope = '%s:%s' % (ParseHelper.getXmlWrapperType(args[0]), args[2])
        actionable_types = TypeHelper.get_actionable_types(
                       expendNestedTypes=True,
                       groupOptions=True
        )

        resource = self.get_object(
                       args[0],
                       args[1],
                       self.resolve_base(opts),
                       context_variants=actionable_types[args[0]]
        )

        if resource is None:
            self.error(
                   Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
            )
        elif hasattr(resource, args[2]) and \
             type(getattr(resource, args[2])) == types.MethodType:
            try:
                result = self.execute_method(resource, args[2], opts)
            except Exception, e:
                self.error(str(e))
            if result.status.state == 'failed':
                self.error(
                       Messages.Info.ACTION_STATUS % result.status.state
                )
        else:
            self.error(
                   Messages.Error.NO_SUCH_ACTION % args[2]
            )
        self.context.formatter.format(self.context, result)

    def show_help(self):
        """Show help for the action command."""
        args = self.arguments
        opts = self.options
        types = TypeHelper.get_actionable_types(
                                expendNestedTypes=True,
                                groupOptions=True
        )
        subst = {}

        if not args or self.is_supported_type(types.keys(), args[0]):
            if len(args) == 2 and len(opts) == 2:
                helptext = self.helptext1

                subst['type'] = args[0]
                subst['id'] = args[1]

                base = self.resolve_base(self.options)
                obj = self.get_object(
                              args[0],
                              args[1],
                              base,
                              context_variants=types[args[0]]
                )
                if obj is None:
                    self.error(
                           Messages.Error.NO_SUCH_OBJECT % (args[1], args[1])
                    )
                actions = self._get_action_methods(obj)
                subst['actions'] = self.format_list(actions)

            if len(args) == 3 and len(opts) == 2:
                helptext = self.helptext1

                subst['type'] = args[0]
                subst['id'] = args[1]
                subst['action'] = args[0]

                base = self.resolve_base(self.options)
                obj = self.get_object(
                              args[1],
                              args[2],
                              base,
                              context_variants=types[args[0]]
                )
                if obj is None:
                    self.error(
                       Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
                    )

                actions = self._get_action_methods(obj)
                if args[0] not in actions:
                    self.error(
                           Messages.Error.NO_SUCH_ACTION % args[2]
                    )

                options = self.get_options(
                           method=args[0],
                           resource=obj,
                           context_variants=types[args[0]]
                )
                subst['actions'] = self.format_list(actions)
                subst['options'] = self.format_list(options, bullet='', sort=False)

            elif len(args) == 1:
                helptext = self.helptext0
                subst['types'] = self.format_map({args[0]:types[args[0]]})
                subst['type'] = args[0]

            elif len(args) == 2:
                helptext = self.helptext1

                subst['type'] = args[0]
                subst['id'] = args[1]
                base = self.resolve_base(opts)
                obj = self.get_object(
                              args[0],
                              args[1],
                              base,
                              context_variants=types[args[0]]
                )
                if obj is None:
                    self.error(
                       Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
                )
                actions = self._get_action_methods(obj)
                subst['actions'] = self.format_list(actions)

            elif len(args) == 3:
                helptext = self.helptext1

                subst['type'] = args[0]
                subst['id'] = args[1]
                subst['action'] = args[2]

                base = self.resolve_base(self.options)
                obj = self.get_object(
                          args[0],
                          args[1],
                          base,
                          context_variants=types[args[0]]
                )
                if obj is None:
                    self.error(
                       Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
                    )

                actions = self._get_action_methods(obj)
                if args[2] not in actions:
                    self.error(Messages.Error.NO_SUCH_ACTION % args[2])

                options = self.get_options(
                           method=args[2],
                           resource=obj,
                           sub_resource=base,
                           context_variants=types[args[0]]
                )
                subst['actions'] = self.format_list(actions)
                subst['options'] = self.format_list(options, bullet='', sort=False)

            else:
                helptext = self.helptext0
                subst['types'] = self.format_map(types)

    #            scope = '%s:%s' % (type(obj).__name__, args[2])

            statuses = self.get_statuses()
            subst['statuses'] = self.format_list(statuses)
            helptext = self.format_help(helptext, subst)
            self.write(helptext)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import sys
import logging
import traceback

from StringIO import StringIO
from subprocess import Popen, PIPE

from cli.error import ParseError, CommandError
from cli.object import create
from cli.settings import Settings
#from cli.parser import Parser
from cli.platform import Terminal
from cli import platform

class ExecutionContext(object):
    """A CLI execution context."""

    name = None

    OK = 0
    SYNTAX_ERROR = 1
    COMMAND_ERROR = 2
    INTERRUPTED = 3
    UNKNOWN_ERROR = 4

#    Parser = Parser
    Terminal = Terminal
    Settings = Settings

    welcome = None
    goodbye = None

    def __init__(self, cmdin=None):
        """Constructor."""
        self.parameters_cash = {}
        self.cmdin = cmdin or sys.stdin
        self.commands = []
        self.status = None
        self.interactive = sys.stdin.isatty() and sys.stdout.isatty() \
                    and self.cmdin is sys.stdin
        self.settings = create(self.Settings, self.name)
#        self.parser = create(self.Parser)
        self.terminal = create(self.Terminal)
        self._setup_logging()
        self._load_settings()
        self.setup_commands()

    def _setup_logging(self):
        """Configure logging."""
        logger = logging.getLogger()
        handler = logging.StreamHandler(sys.stdout)
        formatter = logging.Formatter('%(levelname)s: %(message)s')
        handler.setFormatter(formatter)
        logger.addHandler(handler)
        logger.setLevel(logging.INFO)
        self._logger = logger

    def _load_settings(self):
        """Load settings."""
        found = self.settings.load_config_file()
        if not found:
            self.settings.write_example_config_file()
        self.settings.add_callback('cli:debug', self._set_debug)
        self._set_debug('cli:debug', self.settings['cli:debug'])

    def _set_debug(self, key, value):
        """Enable or disable debugging (callback)."""
        if value:
            level = logging.DEBUG
        else:
            level = logging.INFO
        self._logger.setLevel(level)

    def setup_commands(self):
        """Register the default commands. Override in a subclass."""

    def add_command(self, command):
        """Add an additional command. `command' must implement the Command
        interface."""
        for i in range(len(self.commands)):
            if self.commands[i].name == command.name:
                self.commands[i] = command
                self.parameters_cash[command.name.lower()] = {}
                break
        else:
            self.parameters_cash[command.name.lower()] = {}
            self.commands.append(command)

    def execute_loop(self):
        """Run a read/parse/execute loop until EOF."""
        if self.interactive and self.welcome:
            sys.stdout.write('%s\n' % self.welcome)
        self._eof = False
        while not self._eof:
            command = self._read_command()
            if not command:
                continue
            self.execute_string(command)
        if self.interactive and self.goodbye:
            sys.stdout.write('%s\n' % self.goodbye)

    def execute_string(self, parsed):
#    def execute_string(self, command):
        """Parse and execute a string."""
#        try:
#            parsed = self.parser.parse(command)
#        except EOFError:
#            sys.stderr.write('error: incomplete command\n')
#            return
#        except ParseError, e:
#            self.status = self.SYNTAX_ERROR
#            sys.stderr.write('error: %s\n' % str(e))
#            return
        for command in parsed:
            try:
                self._execute_command(command)
            except Exception, e:
                self._handle_exception(e)
            else:
                self.status = self.OK

    def _handle_exception(self, e):
        """Handle an exception. Can be overruled in a subclass."""
        if isinstance(e, KeyboardInterrupt):
            self.status = self.INTERRUPTED
            sys.stdout.write('\n')
        elif isinstance(e, CommandError):
            self.status = getattr(e, 'status', self.COMMAND_ERROR)
            sys.stderr.write('error: %s\n' % str(e))
            if hasattr(e, 'help'):
                sys.stderr.write('%s\n' % e.help)
        else:
            self.status = self.UNKNOWN_ERROR
            if self.settings['cli:debug']:
                sys.stderr.write(traceback.format_exc())
            else:
                sys.stderr.write('unknown error: %s\n' % str(e))

    def _read_command(self):
        """Parse input until we can parse at least one full command, and
        return that input."""
        command = ''
        prompt = self.settings['cli:ps1']
        while True:
            try:
                if self.interactive:
                    line = self.terminal.readline(prompt)
                else:
                    line = self.cmdin.readline()
            except EOFError:
                self._eof = True
                sys.stdout.write('\n')
                return
            except KeyboardInterrupt:
                sys.stdout.write('\n')
                return
            if not line:
                self._eof = True
                sys.stdout.write('\n')
                return
            command += line
            try:
                parsed = self.parser.parse(command)
            except EOFError:
                prompt = self.settings['cli:ps2']
                continue
            except ParseError, e:
                self.status = self.SYNTAX_ERROR
                sys.stderr.write('error: %s\n' % str(e))
                return
            else:
                break
        return command

    def _execute_command(self, parsed):
        """INTERNAL: execute a command."""
#        if parsed[0] == '!':
#            self._execute_shell_command(parsed[1])
#            return
        #name, args, opts, redirections, pipeline = parsed
        redirections = None
        pipeline = None
        name, args, opts = parsed
        if self.settings.get('cli:autopage'):
            pager = self.settings.get('cli:pager', platform.get_pager())
            if pipeline:
                pipeline += '| %' % pager
            else:
                pipeline = pager
        command = self._create_command(name, args, opts)
        self.command = command
        self._setup_pipeline(pipeline)
        self._setup_io_streams(redirections)
        try:
            command.run(self)
        finally:
            self._restore_io_streams()
            self._execute_pipeline()

    def _execute_shell_command(self, command):
        """INTERNAL: execute a shell command."""
        shell = Popen(command, shell=True)
        retcode = shell.wait()
        if retcode != 0:
            m = 'shell command exited with error'
            raise CommandError(m, status=retcode)

    def _create_command(self, name, args, opts):
        """INTERNAL: instantiate a new command."""
        cls = self._get_command(name)
        if cls is None:
            raise CommandError, 'unknown command: %s' % name
        return cls(args, opts)

    def _get_command(self, name):
        """Return the command class for `name' or None."""
        for cls in self.commands:
            if cls.name == name:
                return cls
            if name in cls.aliases:
                return cls

    def _setup_pipeline(self, pipeline):
        """INTERNAL: set up the pipeline, if any."""
        if not pipeline:
            self._pipeline = None
            return
        self._pipeline = Popen(pipeline, stdin=PIPE, stderr=PIPE, shell=True)
        self._pipeinput = StringIO()
        self.terminal.stdout = self._pipeinput

    def _setup_io_streams(self, redirections=[]):
        """INTERNAL: set up standard input/output/error."""
        for type, arg in redirections:
            if type == '<':
                self.terminal.stdin = file(arg)
            elif type == '<<':
                self.terminal.stdin = StringIO(arg)
            elif type == '>':
                self.terminal.stdout = file(arg, 'w')
            elif type == '>>':
                self.terminal.stdout = file(arg, 'a')

    def _restore_io_streams(self):
        """INTERNAL: reset IO streams."""
        self.terminal.stdin = sys.stdin
        self.terminal.stdout = sys.stdout
        self.terminal.stderr = sys.stderr

    def _execute_pipeline(self):
        """INTERNAL: execute the command pipleine (if any)."""
        if not self._pipeline:
            return
        pipeinput = self._pipeinput.getvalue()
        dummy, stderr = self._pipeline.communicate(pipeinput)
        retcode = self._pipeline.returncode
        if retcode != 0:
            raise CommandError, stderr.rstrip()



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import textwrap

from cli.command import *
from cli.context import ExecutionContext


from ovirtcli.settings import OvirtCliSettings
from ovirtcli.command import *
from ovirtcli.format import *
from ovirtcli.object import create


class OvirtCliExecutionContext(ExecutionContext):

    Settings = OvirtCliSettings

    name = 'ovirt-shell'
    welcome = textwrap.dedent("""\
        Welcome to Red Hat Enterprise Virtualization.

        Type:  'help' for help with commands
               'exit' to leave this interactive shell
        """)

    REMOTE_ERROR = 10
    NOT_FOUND = 11

    def __init__(self):
        super(OvirtCliExecutionContext, self).__init__()
        self.connection = None
        self.formatter = create(Formatter, self.settings['ovirt-shell:output_format'])
        self.settings.add_callback('cli:verbosity', self._set_verbosity)
        self.settings.add_callback('ovirt-shell:output_format', self._set_formatter)

    def _set_verbosity(self, key, value):
        if self.connection is None:
            return
        if value:
            level = 10
        else:
            level = 1
        self.connection.verbosity = level

    def _set_formatter(self, key, value):
        self.formatter = create(Formatter, value)

    def _handle_exception(self, e):
# FIXME: support old error formating
#        if isinstance(e, rhev.Error):
#            msg = 'error: rhev: %s\n' % str(e)
#            if hasattr(e, 'detail'):
#                msg += 'detail: %s\n' % e.detail
#            sys.stderr.write(msg)
#            self.status = self.REMOTE_ERROR
#        else:
#            super(RhevshExecutionContext, self)._handle_exception(e)
        super(OvirtCliExecutionContext, self)._handle_exception(e)

    def setup_commands(self):
        self.add_command(SetCommand)
        self.add_command(SaveCommand)
        self.add_command(CdCommand)
        self.add_command(PwdCommand)
        self.add_command(ClearCommand)
        self.add_command(ExitCommand)
        self.add_command(ActionCommand)
        self.add_command(CreateCommand)
        self.add_command(ConnectCommand)
        self.add_command(ConsoleCommand)
        self.add_command(CreateCommand)
        self.add_command(DeleteCommand)
        self.add_command(DisconnectCommand)
        self.add_command(HelpCommand)
        self.add_command(ListCommand)
        self.add_command(PingCommand)
        self.add_command(ShowCommand)
        self.add_command(StatusCommand)
        self.add_command(UpdateCommand)

    def _get_prompt_variables(self):
        """Return a dict with prompt variables."""
        subst = {}
        if self.connection:
            api = self.connection.api()
            version = api.product_info.version
            subst['version'] = '%s.%s' % (version.major, version.minor)
        else:
            subst['version'] = ''
        return subst

    def _set_prompt(self):
        """Update the prompt."""
        if self.connection is None:
            prompt = self.settings['ovirt-shell:ps1.disconnected']
        else:
#            subst = self._get_prompt_variables()
#            prompt = self.settings['ovirt-shell:ps1.connected'] % subst
            prompt = '[oVirt-shell:connected]# '
        self.settings['cli:ps1'] = prompt

    def _read_command(self):
        self._set_prompt()
        return super(OvirtCliExecutionContext, self)._read_command()
